/* 
 * SPDX-License-Identifier: MIT OR Apache-2.0
 *
 * Copyright (c) 2023 SUSE LLC
 *
 * Author: Roy Hopkins <rhopkins@suse.de>
 */


#ifndef IGVM_H
#define IGVM_H

/* Generated with cbindgen:0.28.0 */

/* This file is generated using cbindgen. Do not edit this file directly. */

#include "igvm_defs.h"

/**
 * Number of bytes in a page for X64.
 */
#define IGVM_X64_PAGE_SIZE 4096

/**
 * Number of bytes in a large page for X64.
 */
#define IGVM_X64_LARGE_PAGE_SIZE 2097152

/**
 * Number of bytes in a 1GB page for X64.
 */
#define IGVM_X64_1GB_PAGE_SIZE 1073741824

enum IgvmHeaderSection {
  IGVM_HEADER_SECTION_PLATFORM,
  IGVM_HEADER_SECTION_INITIALIZATION,
  IGVM_HEADER_SECTION_DIRECTIVE,
};
typedef uint32_t IgvmHeaderSection;

/**
 * An enumeration of the possible results that can be returned from C API
 * functions. Some of the extern "C" functions return a positive value
 * representing a handle or a count on success, or an IgvmResult value on
 * error. Therefore all error values must be negative.
 */
enum IgvmResult {
  IGVMAPI_OK = 0,
  IGVMAPI_INVALID_PARAMETER = -1,
  IGVMAPI_NO_DATA = -2,
  IGVMAPI_INVALID_FILE = -3,
  IGVMAPI_INVALID_HANDLE = -4,
  IGVMAPI_NO_PLATFORM_HEADERS = -5,
  IGVMAPI_FILE_DATA_SECTION_TOO_LARGE = -6,
  IGVMAPI_VARIABLE_HEADER_SECTION_TOO_LARGE = -7,
  IGVMAPI_TOTAL_FILE_SIZE_TOO_LARGE = -8,
  IGVMAPI_INVALID_BINARY_PLATFORM_HEADER = -9,
  IGVMAPI_INVALID_BINARY_INITIALIZATION_HEADER = -10,
  IGVMAPI_INVALID_BINARY_DIRECTIVE_HEADER = -11,
  IGVMAPI_MULTIPLE_PLATFORM_HEADERS_WITH_SAME_ISOLATION = -12,
  IGVMAPI_INVALID_PARAMETER_AREA_INDEX = -13,
  IGVMAPI_INVALID_PLATFORM_TYPE = -14,
  IGVMAPI_NO_FREE_COMPATIBILITY_MASKS = -15,
  IGVMAPI_INVALID_FIXED_HEADER = -16,
  IGVMAPI_INVALID_BINARY_VARIABLE_HEADER_SECTION = -17,
  IGVMAPI_INVALID_CHECKSUM = -18,
  IGVMAPI_MULTIPLE_PAGE_TABLE_RELOCATION_HEADERS = -19,
  IGVMAPI_RELOCATION_REGIONS_OVERLAP = -20,
  IGVMAPI_PARAMETER_INSERT_INSIDE_PAGE_TABLE_REGION = -21,
  IGVMAPI_NO_MATCHING_VP_CONTEXT = -22,
  IGVMAPI_PLATFORM_ARCH_UNSUPPORTED = -23,
  IGVMAPI_INVALID_HEADER_ARCH = -24,
  IGVMAPI_UNSUPPORTED_PAGE_SIZE = -25,
  IGVMAPI_INVALID_FIXED_HEADER_ARCH = -26,
  IGVMAPI_MERGE_REVISION = -27,
};
typedef int32_t IgvmResult;

typedef int32_t IgvmHandle;

/**
 * Returns a pointer to the array of bytes in a buffer.
 *
 * # Safety
 *
 * The caller must ensure that the buffer handle remains valid for the duration
 * of accessing the data pointed to by the raw pointer returned by this
 * function. This requires that the `buffer_handle` is not freed with a call to
 * [`igvm_free_buffer()`] and that the `igvm_handle` is not freed with a call
 * to [`igvm_free()`].
 *
 * Invalid handles are handled within this function and result in a return
 * value of `null()`. The caller must check the result before using the array.
 */
const uint8_t *igvm_get_buffer(IgvmHandle igvm_handle, IgvmHandle buffer_handle);

/**
 * Returns the size of a buffer.
 *
 * If either handle is invalid or if there is an error then the return value will
 * be IGVMAPI_INVALID_HANDLE
 */
int32_t igvm_get_buffer_size(IgvmHandle igvm_handle, IgvmHandle buffer_handle);

/**
 * Frees a buffer.
 *
 * If either handle is invalid then the function has no effect.
 */
void igvm_free_buffer(IgvmHandle igvm_handle, IgvmHandle buffer_handle);

/**
 * Parse a binary array containing an IGVM file. The contents of the file are
 * validated and, if valid, a handle is returned to represent the file. This
 * handle must be freed with a call to igvm_free().
 *
 * If any error occurs then the returned handle will be less than zero and will
 * match one of the IGVMAPI error codes.
 *
 * # Safety
 *
 * The function assumes that there are at least `len` valid bytes of memory
 * starting at the address pointed to by `data`. If this is violated then this
 * will result in undefined behaviour.
 */
IgvmHandle igvm_new_from_binary(const uint8_t *data, uint32_t len);

/**
 * Free a handle that was created with a prevoius call to
 * [`igvm_new_from_binary()`].
 */
void igvm_free(IgvmHandle handle);

/**
 * Get the count of headers for a particular section in a previously parsed
 * IGVM file.
 *
 * If any error occurs then the returned value will be less than zero and will
 * match one of the IGVMAPI error codes.
 */
int32_t igvm_header_count(IgvmHandle handle, IgvmHeaderSection section);

/**
 * Get the header type for the entry with the given index for a particular
 * section in a previously parsed IGVM file.
 *
 * If any error occurs then the returned value will be less than zero and will
 * match one of the IGVMAPI error codes.
 */
int32_t igvm_get_header_type(IgvmHandle handle, IgvmHeaderSection section, uint32_t index);

/**
 * Prepare a buffer containing the header data in binary form for the entry
 * with the given index for a particular section in a previously parsed IGVM
 * file.
 *
 * The buffer containing the data is returned via a handle from this function.
 * The handle can be used to access a raw pointer to the data and to query its
 * size. The buffer handle remains valid until it is closed with a call to
 * [`igvm_free_buffer()`] or the parsed file handle is closed with a call to
 * [`igvm_free()`].
 *
 * If any error occurs then the returned value will be less than zero and will
 * match one of the IGVMAPI error codes.
 */
IgvmHandle igvm_get_header(IgvmHandle handle, IgvmHeaderSection section, uint32_t index);

/**
 * Prepare a buffer containing the associated file data in binary form for the
 * entry with the given index for a particular section in a previously parsed
 * IGVM file.
 *
 * The buffer containing the data is returned via a handle from this function.
 * The handle can be used to access a raw pointer to the data and to query its
 * size. The buffer handle remains valid until it is closed with a call to
 * [`igvm_free_buffer()`] or the parsed file handle is closed with a call to
 * [`igvm_free()`].
 *
 * If any error occurs then the returned value will be less than zero and will
 * match one of the IGVMAPI error codes.
 */
IgvmHandle igvm_get_header_data(IgvmHandle handle, IgvmHeaderSection section, uint32_t index);

#endif  /* IGVM_H */
